@@ -18,9 +18,9 @@
 # the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 
-import os
+import os, popen2
 
-from prewikka import view, User
+from prewikka import view, User, utils
 
 class Error(Exception):
     pass
@@ -49,9 +49,11 @@ class Command(view.View):
 
         command = command.replace("$host", self.parameters["host"]).split(" ")
                 
-        stdin, stdout = os.popen2(command)
-        output = stdout.read()
-        output = output.replace(" ", "&nbsp;").replace("\n", "<br/>\n")
+        pipe = popen2.Popen4(command)
+        pipe.wait()
+
+        output = pipe.fromchild.read()
+        output = utils.escape_html_string(output).replace(" ", "&nbsp;").replace("\n", "<br/>")
         self.dataset["command_output"] = output
 
 
